
/*******************************************************************************

 Copyright (c) 2009, Charles McGarvey
 All rights reserved.
 
 Redistribution   and   use  in  source  and  binary  forms,  with  or  without
 modification, are permitted provided that the following conditions are met:
 
   * Redistributions  of  source  code  must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions  in binary form must reproduce the above copyright notice,
     this  list of conditions and the following disclaimer in the documentation
     and/or other materials provided with the distribution.
 
 THIS  SOFTWARE  IS  PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND  ANY  EXPRESS  OR  IMPLIED  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.  IN  NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 FOR  ANY  DIRECT,  INDIRECT,  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES  (INCLUDING,  BUT  NOT  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES;  LOSS  OF  USE,  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 CAUSED  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 OR  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*******************************************************************************/

#ifndef _MOOF_EXCEPTION_HH_
#define _MOOF_EXCEPTION_HH_

#include <cstring>
#include <exception>
#include <string>

#include <Moof/Math.hh>


namespace Mf {


class Exception : public std::exception
{
public:

	explicit Exception(unsigned code, const std::string& what = "")
	{
		mWhat[sizeof(mWhat)-1] = '\0';
		strncpy(mWhat, what.c_str(), sizeof(mWhat)-1);
		mCode = code;
	}
	virtual ~Exception() throw() {}

	virtual void raise()
	{
		throw *this;
	}

	unsigned code() const throw()
	{
		return mCode;
	}

	const char* what() const throw()
	{
		return mWhat;
	}

private:

	unsigned	mCode;
	char		mWhat[1024];
};

namespace ErrorCode {
enum Code
{
	NONE = 0,
	FASTEVENTS_INIT,			// description
	FILE_NOT_FOUND,				// path of missing file
	RESOURCE_NOT_FOUND,			// name of missing resource
	SCRIPT_ERROR,				// description
	SDL_INIT,					// description
	SDL_VIDEOMODE,				// -
	UNKNOWN_AUDIO_FORMAT,		// name of resource
	UNKNOWN_IMAGE_FORMAT,		// name of resource
};
} // namespace ErrorCode


} // namespace Mf

#endif // _MOOF_EXCEPTION_HH_

/** vim: set ts=4 sw=4 tw=80: *************************************************/

